from abc import ABCMeta, abstractmethod
from typing import Dict, Union

from nidhoggr_core.response import StatusResponse, ErrorResponse
from nidhoggr_core.texture import TextureType, TextureItem
from nidhoggr_core.user import User


class BaseTextureRepo(metaclass=ABCMeta):
    variant: str

    @abstractmethod
    def get(self, *, uuid: str) -> Union[ErrorResponse, Dict[TextureType, TextureItem]]:
        pass


class BaseUserRepo(metaclass=ABCMeta):

    EMPTY_USER = User(
        uuid="00000000000000000000000000000000",
        login="empty",
        email="empty@example.com",
        synthetic=True
    )

    @abstractmethod
    def get_user(self, **kw: Dict[str, str]) -> Union[ErrorResponse, User]:
        pass

    @abstractmethod
    def check_password(self, *, clean: str, uuid: str) -> Union[ErrorResponse, StatusResponse]:
        pass

    @abstractmethod
    def save_user(self, *, user: User) -> Union[ErrorResponse, User]:
        pass



from typing import NamedTuple

from nidhoggr_core.crypto import KeyPair


class BLConfig(NamedTuple):
    key_pair: KeyPair
    strict: bool = False
    simple_login: bool = True
    legacy_compat: bool = True



from typing import Optional, List

from pydantic.main import BaseModel


class UserProperty(BaseModel):
    name: str
    value: str
    signature: Optional[str] = None

    class Config:
        allow_mutation = False
        ignore_extra = False

    def __hash__(self):
        return hash((self.name, self.value, self.signature))

    @property
    def unsigned(self):
        return self.copy(update=dict(signature=None))


class User(BaseModel):
    uuid: str
    login: str
    email: str
    access: Optional[str] = None
    client: Optional[str] = None
    server: Optional[str] = None
    properties: List[UserProperty] = []
    synthetic: bool = False

    class Config:
        allow_mutation = False


class UserRequest(BaseModel):
    uuid: Optional[str] = None
    login: Optional[str] = None
    email: Optional[str] = None
    access: Optional[str] = None
    client: Optional[str] = None
    server: Optional[str] = None

    class Config:
        allow_mutation = False


class PasswordRequest(BaseModel):
    uuid: str
    password: str

    class Config:
        allow_mutation = False
